"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const optimize_client_rects_1 = __importDefault(require("./optimize-client-rects"));
const pdfjs_dom_js_1 = require("./pdfjs-dom.js");
const isClientRectInsidePageRect = (clientRect, pageRect) => {
    if (clientRect.top < pageRect.top) {
        return false;
    }
    if (clientRect.bottom > pageRect.bottom) {
        return false;
    }
    if (clientRect.right > pageRect.right) {
        return false;
    }
    if (clientRect.left < pageRect.left) {
        return false;
    }
    return true;
};
const getPageBorders = (page) => {
    const style = getComputedStyle(page.node);
    return {
        borderTop: parseFloat(style.borderTopWidth.replace('px', '')),
        borderLeft: parseFloat(style.borderLeftWidth.replace('px', '')),
        borderBottom: parseFloat(style.borderBottomWidth.replace('px', '')),
        borderRight: parseFloat(style.borderRightWidth.replace('px', '')),
    };
};
const isMarkedContentEl = (el) => {
    return !!(el === null || el === void 0 ? void 0 : el.className.includes('markedContent'));
};
const getLimitsY = (range) => {
    var _a, _b;
    const limits = {
        start: { y: 0, page: -1 },
        end: { y: 99999, page: -1 }
    };
    const startParentEl = range.startContainer.parentElement;
    const endParentEl = range.endContainer.parentElement;
    const startEl = !isMarkedContentEl(startParentEl)
        ? startParentEl
        : range.startContainer;
    const endEl = !isMarkedContentEl(endParentEl)
        ? endParentEl
        : range.startContainer;
    if ((0, pdfjs_dom_js_1.isHTMLElement)(startEl)) {
        limits.start = {
            y: startEl.offsetTop - startEl.offsetHeight,
            page: ((_a = (0, pdfjs_dom_js_1.getPageFromElement)(startEl)) === null || _a === void 0 ? void 0 : _a.number) || -1
        };
    }
    if ((0, pdfjs_dom_js_1.isHTMLElement)(endEl)) {
        limits.end = {
            y: endEl.offsetTop + endEl.offsetHeight,
            page: ((_b = (0, pdfjs_dom_js_1.getPageFromElement)(endEl)) === null || _b === void 0 ? void 0 : _b.number) || -1
        };
    }
    return limits;
};
const getClientRects = (range, pages, shouldOptimize = true) => {
    const limits = getLimitsY(range);
    const clientRects = Array.from(range.getClientRects());
    const rects = [];
    for (const clientRect of clientRects) {
        for (const page of pages) {
            const pageRect = page.node.getBoundingClientRect();
            const { borderTop, borderBottom, borderLeft, borderRight } = getPageBorders(page);
            if (isClientRectInsidePageRect(clientRect, pageRect) &&
                clientRect.top >= 0 &&
                clientRect.bottom >= 0 &&
                clientRect.width > 0 &&
                clientRect.height > 0 &&
                clientRect.width < pageRect.width - borderLeft - borderRight &&
                clientRect.height < pageRect.height - borderTop - borderBottom) {
                const rect = {
                    top: clientRect.top + page.node.scrollTop - pageRect.top - borderTop,
                    left: clientRect.left + page.node.scrollLeft - pageRect.left - borderLeft,
                    width: clientRect.width,
                    height: clientRect.height,
                    pageNumber: page.number,
                };
                if (rect.top >= borderTop && rect.left >= borderLeft
                    && (page.number === limits.start.page && rect.top >= limits.start.y
                        || page.number !== limits.start.page)
                    && (page.number === limits.end.page && rect.top <= limits.end.y
                        || page.number !== limits.end.page)) {
                    rects.push(rect);
                }
            }
        }
    }
    return shouldOptimize ? (0, optimize_client_rects_1.default)(rects) : rects;
};
exports.default = getClientRects;
//# sourceMappingURL=get-client-rects.js.map